@@ -16,9 +16,14 @@
 
 //! Implementation of the lexer for `FEEL` grammar.
 
-use crate::lalr::TokenType;
-use crate::scope::ParsingScope;
-use {self::errors::*, dmntk_common::errors::Result, dmntk_feel::names::Name};
+use dmntk_feel::context::FeelContext;
+use dmntk_feel::values::VALUE_NULL;
+use {
+  self::errors::*,
+  crate::lalr::TokenType,
+  dmntk_common::Result,
+  dmntk_feel::{Name, Scope},
+};
 
 /// Definition of a single space character.
 const WS: char = ' ';
@@ -41,9 +46,6 @@ pub enum TokenValue {
   Between,
   BetweenAnd,
   Boolean(bool),
-  BuiltInDateTimeFunctionName(String),
-  BuiltInFunctionName(String),
-  BuiltInTypeName(String),
   Colon,
   Comma,
   Context,
@@ -95,22 +97,10 @@ pub enum TokenValue {
   Then,
 }
 
-/// Enumeration indicating how the tokenization of the **name** is processed.
-#[derive(Debug, Copy, Clone, PartialEq)]
-pub enum NameTokenization {
-  // Unchecked,
-// Existing,
-// BuiltInFunctionName,
-// BuiltInTypeName,
-// DateTimeBuiltInFunction,
-// TillInTerminal,
-// KeywordNot,
-}
-
 /// FEEL lexer.
-pub struct Lexer {
+pub struct Lexer<'lexer> {
   /// Parsing scope.
-  scope: ParsingScope,
+  scope: &'lexer Scope,
   /// Starting token type, returned before the first token.
   start_token_type: Option<TokenType>,
   /// Input characters.
@@ -134,9 +124,9 @@ pub struct Lexer {
 }
 
 /// FEEL lexer implementation.
-impl Lexer {
+impl<'lexer> Lexer<'lexer> {
   /// Creates a new lexer for specified input text.
-  pub fn new(scope: ParsingScope, start_token_type: TokenType, input: &str) -> Self {
+  pub fn new(scope: &'lexer Scope, start_token_type: TokenType, input: &str) -> Self {
     Self {
       scope,
       start_token_type: Some(start_token_type),
@@ -148,7 +138,7 @@ impl Lexer {
     }
   }
 
-  pub fn scope(&self) -> &ParsingScope {
+  pub fn scope(&self) -> &Scope {
     &self.scope
   }
 
@@ -165,15 +155,15 @@ impl Lexer {
   }
 
   pub fn push_to_scope(&mut self) {
-    self.scope.push();
+    self.scope.push(FeelContext::default());
   }
 
   pub fn pop_from_scope(&mut self) {
     self.scope.pop();
   }
 
-  pub fn add_name_to_scope(&mut self, name: &str) {
-    self.scope.add_name(name);
+  pub fn add_name_to_scope(&mut self, name: &Name) {
+    self.scope.set_entry(name, VALUE_NULL);
   }
 
   ///
@@ -624,13 +614,14 @@ impl Lexer {
 
     // begin with with the longest name containing all parts
     let mut part_count = parts.len();
+    let flattened_keys = self.scope.flatten_keys();
     while part_count > 0 {
       // take a sublist of the original part list until the list is empty
       let part_sublist = &parts[..part_count];
       // flatten the name parts to compare it with built-in names and keys in current context
       let name = flatten_name_parts(part_sublist);
       // check if the flattened name exists as a key in the current context
-      if self.scope.contains(&name) {
+      if flattened_keys.contains(&name) {
         // return to the input all characters that do not belong to the name that was found
         self.position = consumed_positions[part_count - 1] + 1;
         // return the name that exists in the current context
@@ -1015,7 +1006,7 @@ fn flatten_name_parts(parts: &[String]) -> String {
 
 /// Definitions of errors raised by the lexer.
 pub mod errors {
-  use dmntk_common::errors::DmntkError;
+  use dmntk_common::DmntkError;
 
   /// Lexer errors.
   #[derive(Debug, PartialEq)]
