@@ -16,16 +16,17 @@
 
 //! Implementation of the `LALR` parser for `FEEL` grammar.
 
+use dmntk_feel::context::FeelContext;
+use dmntk_feel::values::VALUE_NULL;
+use dmntk_feel::FeelType;
 use {
   self::errors::*,
   crate::{
-    ast::AstNode,
     lalr::{TokenType, *},
     lexer::{Lexer, TokenValue},
-    scope::ParsingScope,
   },
-  dmntk_common::errors::Result,
-  dmntk_feel::names::Name,
+  dmntk_common::Result,
+  dmntk_feel::{AstNode, Name, Scope},
 };
 
 #[derive(Debug, Clone)]
@@ -51,7 +52,7 @@ macro_rules! trace {
 macro_rules! trace_action {
   ($s:tt, $msg:literal) => {
     if $s.yy_trace {
-      println!("  action: [{}]", $msg);
+      println!("  action: [\u{001b}[32m{}\u{001b}[0m]", $msg);
       println!("  state_stack={:?}", $s.yy_state_stack);
       println!("  value_stack={:?}", $s.yy_value_stack);
       println!("  node_stack={:?}", $s.yy_node_stack);
@@ -60,11 +61,13 @@ macro_rules! trace_action {
 }
 
 /// Parser.
-pub struct Parser {
+pub struct Parser<'parser> {
+  /// Parsing scope.
+  scope: &'parser Scope,
   /// Flag indicating whether the tracing messages should be printed to standard output.
   yy_trace: bool,
   /// The FEEL [Lexer] used by this FEEL [Parser] as an input token stream.
-  yy_lexer: Lexer,
+  yy_lexer: Lexer<'parser>,
   /// The lookahead token type returned by lexer.
   yy_char: i16,
   /// The lookahead semantic value associated with token type returned by lexer.
@@ -85,11 +88,12 @@ pub struct Parser {
   yy_node_stack: Vec<AstNode>,
 }
 
-impl Parser {
+impl<'parser> Parser<'parser> {
   /// Creates a new parser.
-  pub fn new(scope: ParsingScope, start_token_type: TokenType, input: &str, trace: bool) -> Self {
+  pub fn new(scope: &'parser Scope, start_token_type: TokenType, input: &str, trace: bool) -> Self {
     let lexer = Lexer::new(scope, start_token_type, input);
     Self {
+      scope,
       yy_trace: trace,
       yy_lexer: lexer,
       yy_char: TokenType::YyEmpty as i16,
@@ -445,7 +449,7 @@ impl Parser {
     if let Some(value_node) = self.yy_node_stack.pop() {
       if let Some(key_node) = self.yy_node_stack.pop() {
         if let AstNode::ContextEntryKey(name) = &key_node {
-          self.yy_lexer.add_name_to_scope(&name.to_string());
+          self.yy_lexer.add_name_to_scope(&name);
         }
         self.yy_node_stack.push(AstNode::ContextEntry(Box::new(key_node), Box::new(value_node)));
       }
@@ -555,21 +559,40 @@ impl Parser {
     Ok(())
   }
 
+  /// Reduces the function definition.
+  /// The top element on the `node stack` is the AstNode defining the function body.
+  /// The AstNode just below the function's body is the list of function's formal parameters.
   #[inline(always)]
   pub fn action_function_definition(&mut self) -> Result<()> {
     trace_action!(self, "function_definition");
+    if let Some(body) = self.yy_node_stack.pop() {
+      if let Some(parameters) = self.yy_node_stack.pop() {
+        self.yy_node_stack.push(AstNode::FunctionDefinition(Box::new(parameters), Box::new(body)));
+      }
+    }
     Ok(())
   }
 
+  /// Reduces the definition of the function body. This function body is **not** external.
+  /// The content of the function body is the AstNode on the top of the `node stack`.
+  /// After reducing the function body, the top context from scope is popped,
+  /// because this context is temporary and contains the function parameter names
+  /// to be properly interpreted while parsing the function's body.
   #[inline(always)]
   pub fn action_function_body(&mut self) -> Result<()> {
     trace_action!(self, "function_body");
+    if let Some(function_body_node) = self.yy_node_stack.pop() {
+      self.yy_node_stack.push(AstNode::FunctionBody(Box::new(function_body_node), false));
+    }
+    // pop the temporary context from the top of scope stack
+    self.scope.pop();
     Ok(())
   }
 
   #[inline(always)]
   pub fn action_function_body_external(&mut self) -> Result<()> {
     trace_action!(self, "function_body_external");
+    self.scope.pop();
     Ok(())
   }
 
@@ -580,26 +603,73 @@ impl Parser {
   }
 
   #[inline(always)]
-  pub fn action_function_formal_parameter_with_type(&mut self) -> Result<()> {
-    trace_action!(self, "formal_parameter_with_type");
+  pub fn action_formal_parameters_begin(&mut self) -> Result<()> {
+    trace_action!(self, "function_formal_parameters_begin");
+    // when the list of formal parameters begins, push a local context onto scope stack
+    self.scope.push(FeelContext::default());
     Ok(())
   }
 
   #[inline(always)]
-  pub fn action_function_formal_parameter_no_type(&mut self) -> Result<()> {
-    trace_action!(self, "formal_parameter_no_type");
+  pub fn action_formal_parameter_with_type(&mut self) -> Result<()> {
+    trace_action!(self, "function_formal_parameter_with_type");
+    // the type of the parameter is on top of node stack
+    if let Some(rhs) = self.yy_node_stack.pop() {
+      // the name of the parameter is in value stack
+      if let TokenValue::Name(name) = &self.yy_value_stack[self.yy_value_stack.len() - self.yy_len as usize] {
+        // push the new formal parameter on top of node stack
+        self.yy_node_stack.push(AstNode::FormalParameter(name.clone(), rhs.type_of(self.scope)));
+        // set the name of the parameter to local context on the top of the scope stack
+        // this name will be properly interpreted as a name while parsing the function body
+        self.scope.set_entry(name, VALUE_NULL);
+      }
+    }
     Ok(())
   }
 
   #[inline(always)]
-  pub fn action_function_formal_parameters_empty(&mut self) -> Result<()> {
-    trace_action!(self, "formal_parameters_empty");
+  pub fn action_formal_parameter_without_type(&mut self) -> Result<()> {
+    trace_action!(self, "function_formal_parameter_without_type");
+    // the name of the parameter is in value stack
+    if let TokenValue::Name(name) = &self.yy_value_stack[self.yy_value_stack.len() - self.yy_len as usize] {
+      // push the new formal parameter on top of node stack
+      self.yy_node_stack.push(AstNode::FormalParameter(name.clone(), FeelType::Any));
+      // set the name of the parameter to local context on the top of the scope stack
+      // this name will be properly interpreted as a name while parsing the function body
+      self.scope.set_entry(name, VALUE_NULL);
+    }
     Ok(())
   }
 
   #[inline(always)]
-  pub fn action_function_formal_parameters_tail(&mut self) -> Result<()> {
-    trace_action!(self, "formal_parameters_tail");
+  pub fn action_formal_parameters_empty(&mut self) -> Result<()> {
+    trace_action!(self, "function_formal_parameters_empty");
+    // push the empty list of formal parameters onto the node stack
+    self.yy_node_stack.push(AstNode::FormalParameters(vec![]));
+    Ok(())
+  }
+
+  #[inline(always)]
+  pub fn action_formal_parameters_first(&mut self) -> Result<()> {
+    trace_action!(self, "function_formal_parameters_first");
+    // the first parameter is on the top of the node stack
+    if let Some(formal_parameter_node) = self.yy_node_stack.pop() {
+      self.yy_node_stack.push(AstNode::FormalParameters(vec![formal_parameter_node]));
+    }
+    Ok(())
+  }
+
+  #[inline(always)]
+  pub fn action_formal_parameters_tail(&mut self) -> Result<()> {
+    trace_action!(self, "function_formal_parameters_tail");
+    // the next parameter is on the top of the node stack
+    if let Some(formal_parameter_node) = self.yy_node_stack.pop() {
+      // the collection of formal parameters is now on top of the node stack
+      if let Some(AstNode::FormalParameters(mut items)) = self.yy_node_stack.pop() {
+        items.push(formal_parameter_node);
+        self.yy_node_stack.push(AstNode::FormalParameters(items));
+      }
+    }
     Ok(())
   }
 
@@ -758,8 +828,8 @@ impl Parser {
   #[inline(always)]
   pub fn action_key_name(&mut self) -> Result<()> {
     trace_action!(self, "key_name");
-    if let Some(TokenValue::Name(value)) = self.yy_value_stack.last() {
-      self.yy_node_stack.push(AstNode::ContextEntryKey(value.clone()));
+    if let Some(TokenValue::Name(name)) = self.yy_value_stack.last() {
+      self.yy_node_stack.push(AstNode::ContextEntryKey(name.clone()));
     }
     Ok(())
   }
@@ -1019,7 +1089,7 @@ impl Parser {
 
 /// Definitions of errors raised by the parser.
 pub mod errors {
-  use dmntk_common::errors::DmntkError;
+  use dmntk_common::DmntkError;
 
   /// Parser errors.
   #[derive(Debug, PartialEq)]
