@@ -14,6 +14,7 @@
  * SOFTWARE.
  */
 
+use dmntk_feel::values::VALUE_NULL;
 use {
   super::{accept, empty_scope},
   crate::lalr::TokenType::StartTextualExpression,
@@ -22,7 +23,7 @@ use {
 #[test]
 fn test_instance_of_1() {
   let scope = &mut empty_scope();
-  scope.add_name("Person");
+  scope.set_entry(&"Person".into(), VALUE_NULL);
   accept(
     scope,
     StartTextualExpression,
@@ -40,7 +41,7 @@ fn test_instance_of_1() {
 #[test]
 fn test_instance_of_2() {
   let scope = &mut empty_scope();
-  scope.add_name("Person");
+  scope.set_entry(&"Person".into(), VALUE_NULL);
   accept(
     scope,
     StartTextualExpression,
